/* This file contains commands to create all the Bank tables */

-- Removes tables if previously created 
-- If the table has NOT been created, the DROP command will fail (this is OK)
-- CASCADE CONSTRAINTS deletes the table even if other tables have foreign keys
-- which reference this table 
DROP TABLE Owner CASCADE CONSTRAINTS
go
DROP TABLE Customer CASCADE CONSTRAINTS
go
DROP TABLE Account CASCADE CONSTRAINTS
go
DROP TABLE Employee CASCADE CONSTRAINTS
go
DROP TABLE Branch CASCADE CONSTRAINTS
go

--Creates tables. Take care to create tables in correct order.
--e.g.Create Branch before Account, which references Branch number
--	Create Customer before Owner, which references Customer id 
--	etc.
--	If 2 tables both reference each other, you will need to add one constraint
--	later using ALTER TABLE
CREATE TABLE Branch (
	branchNo	NUMBER(2)		CONSTRAINT pk_Branch PRIMARY KEY,
	brAddress	VARCHAR2(40)	CONSTRAINT nn_add NOT NULL,
	manager	NUMBER(3)		
)
go
CREATE TABLE Customer (
	id		NUMBER(3)		CONSTRAINT 	ck_cust_id	CHECK	(id between 100 and 999)	
						CONSTRAINT 	pk_customer	PRIMARY KEY,
	forename	VARCHAR2(15)	CONSTRAINT nn_cfn NOT NULL,
	surname	VARCHAR2(15)	CONSTRAINT nn_csn NOT NULL,
	sex		CHAR(1)		CONSTRAINT 	ck_cust_sex	CHECK (sex in ('M', 'F')),
	address	VARCHAR2(30),
	occupation	VARCHAR2(15)	
 )
go
CREATE TABLE Account (
	accountNo	NUMBER(5)		CONSTRAINT	pk_Account	PRIMARY KEY,
	type		VARCHAR2(10)	CONSTRAINT nn_type NOT NULL,
	balance	NUMBER(20,2) 	CONSTRAINT nn_bal NOT NULL,
	dateOpened	DATE,			
	inBranch	NUMBER(2) 		CONSTRAINT nn_abr NOT NULL
						CONSTRAINT	fk_ABranch REFERENCES Branch(branchNo)
)
go
-- Note composite primary key must be specified at end of column declarations
-- Foreign key is specified at end just to show slightly different syntax
CREATE TABLE Owner (
	accNo		NUMBER(5),
	custId	NUMBER(3),
	CONSTRAINT 	pk_Owner	PRIMARY KEY (accNO, custId),
	CONSTRAINT	fk_OAcc	FOREIGN KEY (accNo) REFERENCES Account(accountNo),
	CONSTRAINT	fk_OCust	FOREIGN KEY (custId) REFERENCES Customer(id)
)
go
CREATE TABLE Employee (
	staffNo	NUMBER(3)		CONSTRAINT	pk_emp PRIMARY KEY
						CONSTRAINT	ck_StaffNo	CHECK (staffNo > 100),
	foreName	VARCHAR2(20) 	CONSTRAINT	pk_efn NOT NULL,
	surname	VARCHAR2(20)	CONSTRAINT	pk_esn NOT NULL,
	empBranch	NUMBER(2)		CONSTRAINT	pk_ebr NOT NULL
						CONSTRAINT	fk_EBranch REFERENCES Branch(branchNo),
	supervisor	NUMBER(3)		CONSTRAINT 	fk_Super REFERENCES Employee(staffNo)
)
go
-- Adds Foreign Key constraint to Branch table, now that Employee table has been created.
ALTER TABLE BRANCH 
ADD (
	CONSTRAINT	fk_Manager	FOREIGN KEY (manager) REFERENCES Employee(staffNo)
)
go